@@ -2,13 +2,13 @@ using System;
 using System.Reflection;
 using System.Runtime.CompilerServices;
 
-[assembly: ApplicationVersion ("2.9.0", "2.9")]
-[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "x86_64", "2013-10-08 13:08:33 CEST")]
+[assembly: ApplicationVersion ("2.9.1", "2.9")]
+[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "x86_64", "2014-03-18 16:53:30 CET")]
 
 [assembly: AssemblyVersion ("2.9.0.0")]
 [assembly: AssemblyTitle ("Banshee")]
 [assembly: AssemblyDescription ("Banshee Media Player")]
-[assembly: AssemblyCopyright ("Copyright (C) 2005-2010 Novell Inc. and others")]
+[assembly: AssemblyCopyright ("Copyright (C) 2005-2014 Novell Inc. and others")]
 [assembly: AssemblyCompany ("Novell, Inc.")]
 
 [AttributeUsage (AttributeTargets.Assembly, Inherited = false)]
