@@ -423,7 +423,7 @@ namespace Banshee.GStreamer
 
                 // If the state is anything other than loaded, assume we were just playing a track and should
                 // EoS it and increment its playcount etc.
-                if (CurrentState != PlayerState.Loaded) {
+                if (CurrentState != PlayerState.Loaded && CurrentState != PlayerState.Loading) {
                     ServiceManager.PlayerEngine.IncrementLastPlayed (1.0);
                     OnEventChanged (PlayerEvent.EndOfStream);
                     OnEventChanged (PlayerEvent.StartOfStream);
@@ -431,8 +431,19 @@ namespace Banshee.GStreamer
             }
         }
 
+        private System.DateTime about_to_finish_time_stamp;
+
         private void OnAboutToFinish (IntPtr player)
         {
+            // HACK: ugly workaround for GStreamer's bug http://bugzilla.gnome.org/722769
+            // long story short, AboutToFinish signal firing twice for the same play of the same track
+            // causes problems when Gapless Enabled because of RequestNextTrack event being fired twice
+            if (about_to_finish_time_stamp == CurrentTrackTimeStamp) {
+                return;
+            }
+            about_to_finish_time_stamp = CurrentTrackTimeStamp;
+
+
             // This is needed to make Shuffle-by-* work.
             // Shuffle-by-* uses the LastPlayed field to determine what track in the grouping to play next.
             // Therefore, we need to update this before requesting the next track.
@@ -750,16 +761,11 @@ namespace Banshee.GStreamer
             get { return gapless_enabled; }
             set
             {
-                if (bp_supports_gapless (handle)) {
-                    gapless_enabled = value;
-                    if (value) {
-                        bp_set_about_to_finish_callback (handle, about_to_finish_callback);
-                    } else {
-                        bp_set_about_to_finish_callback (handle, null);
-                    }
+                gapless_enabled = value;
+                if (value) {
+                    bp_set_about_to_finish_callback (handle, about_to_finish_callback);
                 } else {
-                    gapless_enabled = false;
-                    next_track_pending = false;
+                    bp_set_about_to_finish_callback (handle, null);
                 }
             }
         }
@@ -884,13 +890,11 @@ namespace Banshee.GStreamer
                 Catalog.GetString ("For tracks that have ReplayGain data, automatically scale (normalize) playback volume"),
                 delegate { ReplayGainEnabled = ReplayGainEnabledSchema.Get (); }
             ));
-            if (bp_supports_gapless (handle)) {
-                gapless_preference = service["general"]["misc"].Add (new SchemaPreference<bool> (GaplessEnabledSchema,
-                        Catalog.GetString ("Enable _gapless playback"),
-                        Catalog.GetString ("Eliminate the small playback gap on track change. Useful for concept albums and classical music"),
-                        delegate { GaplessEnabled = GaplessEnabledSchema.Get (); }
-                ));
-            }
+            gapless_preference = service["general"]["misc"].Add (new SchemaPreference<bool> (GaplessEnabledSchema,
+                    Catalog.GetString ("Enable _gapless playback"),
+                    Catalog.GetString ("Eliminate the small playback gap on track change. Useful for concept albums and classical music"),
+                    delegate { GaplessEnabled = GaplessEnabledSchema.Get (); }
+            ));
         }
 
         private void UninstallPreferences ()
@@ -901,9 +905,7 @@ namespace Banshee.GStreamer
             }
 
             service["general"]["misc"].Remove (replaygain_preference);
-            if (bp_supports_gapless (handle)) {
-                service["general"]["misc"].Remove (gapless_preference);
-            }
+            service["general"]["misc"].Remove (gapless_preference);
             replaygain_preference = null;
             gapless_preference = null;
         }
@@ -971,9 +973,6 @@ namespace Banshee.GStreamer
         private static extern void bp_set_about_to_finish_callback (HandleRef player,
             BansheePlayerAboutToFinishCallback cb);
 
-        [DllImport ("libbanshee.dll")]
-        private static extern bool bp_supports_gapless (HandleRef player);
-
         [DllImport ("libbanshee.dll")]
         private static extern bool bp_open (HandleRef player, IntPtr uri, bool maybeVideo);
 
