@@ -1,10 +1,14 @@
 //
 // PlayerEngine.cs
 //
-// Author:
+// Authors:
 //   Gabriel Burt <gburt@novell.com>
+//   Andrés G. Aragoneses <knocte@gmail.com>
+//   Stephan Sundermann <stephansundermann@gmail.com>
 //
 // Copyright (C) 2010 Novell, Inc.
+// Copyright (C) 2013-2014 Andrés G. Aragoneses
+// Copyright (C) 2013 Stephan Sundermann
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -29,34 +33,27 @@
 using System;
 using System.Collections;
 using System.Collections.Generic;
-using System.Linq;
-using System.Runtime.InteropServices;
 using System.Threading;
 
 using Mono.Unix;
 
 using Gst;
 using Gst.PbUtils;
-using Gst.BasePlugins;
-using Gst.CorePlugins;
-using Gst.Interfaces;
 
 using Hyena;
-using Hyena.Data;
-
-using Banshee.Base;
 using Banshee.Collection;
 using Banshee.Streaming;
 using Banshee.MediaEngine;
 using Banshee.ServiceStack;
 using Banshee.Configuration;
 using Banshee.Preferences;
+using Gst.Video;
 
 namespace Banshee.GStreamerSharp
 {
     public class PlayerEngine : Banshee.MediaEngine.PlayerEngine, IEqualizer, IVisualizationDataSource
     {
-        private class AudioSinkBin : Bin
+        internal class AudioSinkBin : Bin
         {
             Element hw_audio_sink;
             Element volume;
@@ -65,7 +62,7 @@ namespace Banshee.GStreamerSharp
             Element preamp;
             Element first;
             GhostPad visible_sink;
-            Tee audiotee;
+            Element audiotee;
             object pipeline_lock = new object ();
 
             public AudioSinkBin (IntPtr o) : base(o)
@@ -80,7 +77,7 @@ namespace Banshee.GStreamerSharp
                 first = hw_audio_sink;
 
                 // Our audio sink is a tee, so plugins can attach their own pipelines
-                audiotee = ElementFactory.Make ("tee", "audiotee") as Tee;
+                audiotee = ElementFactory.Make ("tee", "audiotee");
                 if (audiotee == null) {
                     Log.Error ("Can not create audio tee!");
                 } else {
@@ -101,12 +98,14 @@ namespace Banshee.GStreamerSharp
                 }
 
                 equalizer = ElementFactory.Make ("equalizer-10bands", "equalizer-10bands");
+
                 if (equalizer != null) {
                     Element eq_audioconvert = ElementFactory.Make ("audioconvert", "audioconvert");
                     Element eq_audioconvert2 = ElementFactory.Make ("audioconvert", "audioconvert2");
                     preamp = ElementFactory.Make ("volume", "preamp");
 
-                    Add (eq_audioconvert, preamp, equalizer, eq_audioconvert2);
+                    this.Add (eq_audioconvert, preamp, equalizer, eq_audioconvert2);
+
                     Element.Link (eq_audioconvert, preamp, equalizer, eq_audioconvert2, first);
 
                     first = eq_audioconvert;
@@ -115,8 +114,8 @@ namespace Banshee.GStreamerSharp
 
                 // Link the first tee pad to the primary audio sink queue
                 Pad sinkpad = first.GetStaticPad ("sink");
-                Pad pad = audiotee.GetRequestPad ("src%d");
-                audiotee.AllocPad = pad;
+                Pad pad = RequestTeePad ();
+                audiotee["alloc-pad"] = pad;
                 pad.Link (sinkpad);
                 first = audiotee;
 
@@ -132,18 +131,21 @@ namespace Banshee.GStreamerSharp
                 // the Ready state before they'll contain an actual sink.
                 sink.SetState (State.Ready);
 
-                if (sink.HasProperty ("volume")) {
+                try {
+                    var volume = sink ["volume"];
                     volumeProvider = sink;
-                    Log.DebugFormat ("Sink {0} has native volume.", volumeProvider.Name);
-                } else {
+                    Log.DebugFormat ("Sink {0} has native volume: {1}", volumeProvider.Name, volume);
+                } catch (Gst.PropertyNotFoundException) {
                     var sinkBin = sink as Bin;
                     if (sinkBin != null) {
-                        foreach (Element e in sinkBin.ElementsRecurse) {
-                            if (e.HasProperty ("volume")) {
+                        foreach (Element e in sinkBin.IterateRecurse ()) {
+                            try {
+                                var volume = e ["volume"];
                                 volumeProvider = e;
-                                Log.DebugFormat ("Found volume provider {0} in {1}.",
-                                    volumeProvider.Name, sink.Name);
-                            }
+                                Log.DebugFormat ("Found volume provider {0} in {1}: {2}",
+                                    volumeProvider.Name, sink.Name, volume);
+                                break;
+                            } catch (Gst.PropertyNotFoundException) { }
                         }
                     }
                 }
@@ -178,16 +180,16 @@ namespace Banshee.GStreamerSharp
                 get { return rgvolume != null; }
                 set {
                     if (value && rgvolume == null) {
-                        visible_sink.SetBlocked (true, InsertReplayGain);
+                        visible_sink.AddProbe (PadProbeType.Idle, InsertReplayGain);
                         Log.Debug ("Enabled ReplayGain volume scaling.");
                     } else if (!value && rgvolume != null) {
-                        visible_sink.SetBlocked (false, RemoveReplayGain);
+                        visible_sink.AddProbe (PadProbeType.Idle, RemoveReplayGain);
                         Log.Debug ("Disabled ReplayGain volume scaling.");
                     }
                 }
             }
 
-            void InsertReplayGain (Pad pad, bool blocked)
+            PadProbeReturn InsertReplayGain (Pad pad, PadProbeInfo info)
             {
                 lock (pipeline_lock) {
                     if (rgvolume == null) {
@@ -199,10 +201,10 @@ namespace Banshee.GStreamerSharp
                         first = rgvolume;
                     }
                 }
-                visible_sink.SetBlocked (false, (_, __) => { });
+                return PadProbeReturn.Remove;
             }
 
-            void RemoveReplayGain (Pad pad, bool blocked)
+            PadProbeReturn RemoveReplayGain (Pad pad, PadProbeInfo info)
             {
                 lock (pipeline_lock) {
                     if (rgvolume != null) {
@@ -214,7 +216,7 @@ namespace Banshee.GStreamerSharp
                         visible_sink.SetTarget (first.GetStaticPad ("sink"));
                     }
                 }
-                visible_sink.SetBlocked (false, (_, __) => { });
+                return PadProbeReturn.Remove;
             }
 
 
@@ -238,73 +240,48 @@ namespace Banshee.GStreamerSharp
                 set { preamp ["volume"] = Math.Pow (10.0, value / 20.0); }
             }
 
-            public int [] BandRange {
-                get {
-                    int min = -1;
-                    int max = -1;
-
-                    PropertyInfo pspec = new PropertyInfo();
-
-                    if (equalizer.HasProperty ("band0::gain")) {
-                        pspec = equalizer.GetPropertyInfo ("band0::gain");
-                    } else if (equalizer.HasProperty ("band0")) {
-                        pspec = equalizer.GetPropertyInfo ("band0");
-                    }
-
-                    if (pspec.Name != null) {
-                        min = (int)((double)pspec.Min);
-                        max = (int)((double)pspec.Max);
-                    }
-
-                    return new int [] { min, max };
-                }
-            }
+            // The gain range is the same for all bands, and hardcoded in the "equalizer-10bands" element
+            // http://gstreamer.freedesktop.org/data/doc/gstreamer/head/gst-plugins-good-plugins/html/gst-plugins-good-plugins-equalizer-10bands.html#GstIirEqualizer10Bands--band0
+            private int [] band_range = new int [] { -24, 12 };
+            public int [] BandRange { get { return band_range; } }
 
             private uint GetNBands ()
             {
                 if (equalizer == null) {
                     return 0;
-                }
-
-                return ChildProxyAdapter.GetObject (equalizer).ChildrenCount;
-            }
-
-            public uint [] EqualizerFrequencies {
-                get {
-                    uint count = GetNBands ();
-                    uint[] ret = new uint[count];
-
-                    if (equalizer != null) {
-                        ChildProxy equalizer_child_proxy = ChildProxyAdapter.GetObject (equalizer);
-                        for (uint i = 0; i < count; i++) {
-                            Gst.Object band = equalizer_child_proxy.GetChildByIndex (i);
-                            ret [i] = (uint)(double)band ["freq"];
-                        }
-                    }
-
-                    return ret;
+                } else {
+                    // We're using "equalizer-10bands" which always has 10 bands
+                    return 10;
                 }
             }
 
+            // Frequency band values are hardcoded in the "equalizer-10bands" element
+            // See http://gstreamer.freedesktop.org/data/doc/gstreamer/head/gst-plugins-good-plugins/html/gst-plugins-good-plugins-equalizer-10bands.html#gst-plugins-good-plugins-equalizer-10bands.property-details
+            private uint [] frequencies = new uint [] { 29, 59, 119, 237, 474, 947, 1889, 3770, 7523, 15011 }; 
+            public uint [] EqualizerFrequencies { get { return frequencies; } }
+
             public void SetEqualizerGain (uint band, double value)
             {
                 if (equalizer != null) {
                     if (band >= GetNBands ()) {
                         throw new ArgumentOutOfRangeException ("band", "Attempt to set out-of-range equalizer band");
                     }
-                    Gst.Object the_band = ChildProxyAdapter.GetObject (equalizer).GetChildByIndex (band);
-                    the_band ["gain"] = value;
+                    equalizer ["band" + band] = value;
                 }
             }
 
             public Pad RequestTeePad ()
             {
-                return audiotee.GetRequestPad ("src%d");
+                var pad = audiotee.GetRequestPad ("src_%u");
+                if (pad == null) {
+                    throw new InvalidOperationException ("Could not retrieve tee pad");
+                }
+                return pad;
             }
         }
 
 
-        PlayBin2 playbin;
+        Element playbin;
         AudioSinkBin audio_sink;
         uint iterate_timeout_id = 0;
         List<string> missing_details = new List<string> ();
@@ -314,6 +291,11 @@ namespace Banshee.GStreamerSharp
         DvdManager dvd_manager = null;
         Visualization visualization;
 
+        bool gapless_enabled;
+        private bool next_track_pending;
+        private SafeUri pending_uri;
+        private bool pending_maybe_video;
+
         public PlayerEngine ()
         {
             Log.InformationFormat ("GStreamer# {0} Initializing; {1}.{2}",
@@ -338,7 +320,18 @@ namespace Banshee.GStreamerSharp
             }
 
             Gst.Application.Init ();
-            playbin = new PlayBin2 ();
+
+            var profile_manager = ServiceManager.MediaProfileManager;
+            if (profile_manager != null) {
+                profile_manager.Initialized += MediaProfileBackend.OnMediaProfileManagerInitialized;
+            }
+        }
+
+        protected override bool DelayedInitialize { get { return true; } }
+
+        protected override void Initialize ()
+        {
+            playbin = ElementFactory.Make ("playbin", "the playbin");
 
             next_track_set = new ManualResetEvent (false);
 
@@ -351,35 +344,25 @@ namespace Banshee.GStreamerSharp
                 Volume = (ushort)PlayerEngineService.VolumeSchema.Get ();
             }
 
-            Pad teepad = audio_sink.RequestTeePad ();
-            visualization = new Visualization (audio_sink, teepad);
+            visualization = new Visualization (audio_sink);
 
             playbin.AddNotification ("volume", OnVolumeChanged);
             playbin.Bus.AddWatch (OnBusMessage);
-            playbin.AboutToFinish += OnAboutToFinish;
 
             cdda_manager = new CddaManager (playbin);
             dvd_manager = new DvdManager (playbin);
-            // FIXME: Disable video stuff until GLib# 3 is used instead of the sopy bundled in GStreamerSharp
-            //video_manager = new VideoManager (playbin);
-            //video_manager.PrepareWindow += OnVideoPrepareWindow;
-            //video_manager.Initialize ();
+
+            video_manager = new VideoManager (playbin);
+            video_manager.PrepareWindow += OnVideoPrepareWindow;
+            video_manager.Initialize ();
 
             dvd_manager.FindNavigation (playbin);
+
             OnStateChanged (PlayerState.Ready);
-        }
 
-        protected override bool DelayedInitialize {
-            get {
-                return true;
-            }
-        }
-
-        protected override void Initialize ()
-        {
-            base.Initialize ();
             InstallPreferences ();
             audio_sink.ReplayGainEnabled = ReplayGainEnabledSchema.Get ();
+            GaplessEnabled = GaplessEnabledSchema.Get ();
         }
 
         public override void Dispose ()
@@ -413,8 +396,19 @@ namespace Banshee.GStreamerSharp
             OnEventChanged (PlayerEvent.PrepareVideoWindow);
         }
 
-        void OnAboutToFinish (object o, Gst.GLib.SignalArgs args)
+        private System.DateTime about_to_finish_time_stamp;
+
+        void OnAboutToFinish (object o, GLib.SignalArgs args)
         {
+            // HACK: ugly workaround for GStreamer's bug http://bugzilla.gnome.org/722769
+            // long story short, AboutToFinish signal firing twice for the same play of the same track
+            // causes problems when Gapless Enabled because of RequestNextTrack event being fired twice
+            if (about_to_finish_time_stamp == CurrentTrackTimeStamp) {
+                return;
+            }
+            about_to_finish_time_stamp = CurrentTrackTimeStamp;
+
+
             // This is needed to make Shuffle-by-* work.
             // Shuffle-by-* uses the LastPlayed field to determine what track in the grouping to play next.
             // Therefore, we need to update this before requesting the next track.
@@ -424,8 +418,13 @@ namespace Banshee.GStreamerSharp
             CurrentTrack.UpdateLastPlayed ();
 
             next_track_set.Reset ();
-            OnEventChanged (PlayerEvent.RequestNextTrack);
+            pending_uri = null;
+            next_track_pending = true;
 
+            OnEventChanged (PlayerEvent.RequestNextTrack);
+            // Gapless playback with Playbin2 requires that the about-to-finish callback does not return until
+            // the next uri has been set.  Block here for a second or until the RequestNextTrack event has
+            // finished triggering.
             if (!next_track_set.WaitOne (1000, false)) {
                 Log.Warning ("[Gapless]: Timed out while waiting for next track to be set.");
                 next_track_set.Set ();
@@ -434,13 +433,23 @@ namespace Banshee.GStreamerSharp
 
         public override void SetNextTrackUri (SafeUri uri, bool maybeVideo)
         {
+            next_track_pending = false;
             if (next_track_set.WaitOne (0, false)) {
                 // We've been asked to set the next track, but have taken too
                 // long to get here.  Bail for now, and the EoS handling will
                 // pick up the pieces.
+                pending_uri = uri;
+                pending_maybe_video = maybeVideo;
                 return;
             }
-            playbin.Uri = uri.AbsoluteUri;
+
+            // If there isn't a next track for us, release the block on the about-to-finish callback.
+            if (uri == null) {
+                next_track_set.Set ();
+                return;
+            }
+
+            playbin ["uri"] = uri.AbsoluteUri;
 
             if (maybeVideo) {
                 LookupForSubtitle (uri);
@@ -455,11 +464,37 @@ namespace Banshee.GStreamerSharp
             if (accurate_seek) {
                 seek_flags |= SeekFlags.Accurate;
             }
-               
-            playbin.Seek (Format.Time, seek_flags, (long)(position * Gst.Clock.MSecond));
+            playbin.SeekSimple (Format.Time, seek_flags, (((long)position) * Constants.MSECOND));
             OnEventChanged (PlayerEvent.Seek);
         }
 
+        private void OnEos ()
+        {
+            if (!next_track_pending &&
+                (!GaplessEnabled || (CurrentTrack != null && CurrentTrack.HasAttribute (TrackMediaAttributes.VideoStream)))) {
+                // We don't request next track in OnEoS if gapless playback is enabled and current track has no video stream contained.
+                // The request next track is already called in OnAboutToFinish().
+                OnEventChanged (PlayerEvent.RequestNextTrack);
+            } else if (pending_uri != null) {
+                Log.Warning ("[Gapless] EOS signalled while waiting for next track.  This means that Banshee " +
+                    "was too slow at calculating what track to play next.  " +
+                    "If this happens frequently, please file a bug");
+                OnStateChanged (PlayerState.Loading);
+                OpenUri (pending_uri, pending_maybe_video);
+                Play ();
+                pending_uri = null;
+            } else if (!GaplessEnabled || (CurrentTrack != null && CurrentTrack.HasAttribute (TrackMediaAttributes.VideoStream))) {
+                // This should be unreachable - the RequestNextTrack event is delegated to the main thread
+                // and so blocks the bus callback from delivering the EOS message.
+                //
+                // Playback should continue as normal from here, when the RequestNextTrack message gets handled.
+                Log.Warning ("[Gapless] EndOfStream message received before the next track has been set.  " +
+                    "If this happens frequently, please file a bug");
+            } else {
+                Log.Debug ("[Gapless] Reach the last music under repeat off mode");
+            }
+        }
+
         private bool OnBusMessage (Bus bus, Message msg)
         {
             switch (msg.Type) {
@@ -467,7 +502,8 @@ namespace Banshee.GStreamerSharp
                     StopIterating ();
                     Close (false);
                     OnEventChanged (PlayerEvent.EndOfStream);
-                    OnEventChanged (PlayerEvent.RequestNextTrack);
+
+                    OnEos ();
                     break;
 
                 case MessageType.StateChanged:
@@ -479,30 +515,27 @@ namespace Banshee.GStreamerSharp
                     break;
 
                 case MessageType.Buffering:
-                    int buffer_percent;
-                    msg.ParseBuffering (out buffer_percent);
+                    int buffer_percent = msg.ParseBuffering ();
                     HandleBuffering (buffer_percent);
                     break;
 
                 case MessageType.Tag:
-                    Pad pad;
-                    TagList tag_list;
-                    msg.ParseTag (out pad, out tag_list);
-
-                    HandleTag (pad, tag_list);
+                    TagList tag_list = msg.ParseTag ();
+                    tag_list.Foreach (HandleTag);
                     break;
 
                 case MessageType.Error:
-                    Enum error_type;
-                    string err_msg, debug;
-                    msg.ParseError (out error_type, out err_msg, out debug);
+                    GLib.GException err;
+                    string debug;
+                    msg.ParseError (out err, out debug);
 
-                    HandleError (error_type, err_msg, debug);
+                    HandleError (err);
                     break;
 
                 case MessageType.Element:
-                    if (MissingPluginMessage.IsMissingPluginMessage (msg)) {
-                        string detail = MissingPluginMessage.GetInstallerDetail (msg);
+
+                    if (GlobalPbUtil.IsMissingPluginMessage (msg)) {
+                        string detail = GlobalPbUtil.MissingPluginMessageGetInstallerDetail (msg);
 
                         if (detail == null)
                             return false;
@@ -515,16 +548,19 @@ namespace Banshee.GStreamerSharp
                         Log.DebugFormat ("Saving missing element details ('{0}')", detail);
                         missing_details.Add (detail);
 
-                        Log.Error ("Missing GStreamer Plugin", MissingPluginMessage.GetDescription (msg), true);
+                        Log.Error ("Missing GStreamer Plugin", GlobalPbUtil.MissingPluginMessageGetDescription (msg), true);
 
                         InstallPluginsContext install_context = new InstallPluginsContext ();
-                        Install.InstallPlugins (missing_details.ToArray (), install_context, OnInstallPluginsReturn);
-                    } else if (msg.Src == playbin && msg.Structure.Name == "playbin2-stream-changed") {
-                        HandleStreamChanged ();
-                    } else if (NavigationMessage.MessageGetType (msg) == NavigationMessageType.CommandsChanged) {
+                        GlobalPbUtil.InstallPluginsAsync (missing_details.ToArray (), install_context, OnInstallPluginsReturn);
+                    } else if (NavigationAdapter.MessageGetType (msg) == NavigationMessageType.CommandsChanged) {
                         dvd_manager.HandleCommandsChanged (playbin);
                     }
                     break;
+
+                case MessageType.StreamStart:
+                    HandleStreamStart ();
+                    break;
+
                 case MessageType.Application:
                     string name;
                     Structure s = msg.Structure;
@@ -545,28 +581,38 @@ namespace Banshee.GStreamerSharp
             }
         }
 
-        private void OnVolumeChanged (object o, Gst.GLib.NotifyArgs args)
+        private void OnVolumeChanged (object o, GLib.NotifyArgs args)
         {
             OnEventChanged (PlayerEvent.Volume);
         }
 
-        private void HandleStreamChanged ()
+        private void HandleStreamStart ()
         {
-            // Set the current track as fully played before signaling EndOfStream.
-            ServiceManager.PlayerEngine.IncrementLastPlayed (1.0);
-            OnEventChanged (PlayerEvent.EndOfStream);
-            OnEventChanged (PlayerEvent.StartOfStream);
+            if (!GaplessEnabled) {
+                return;
+            }
+
+            // Must do it here because the next track is already playing.
+
+            // If the state is anything other than loaded, assume we were just playing a track and should
+            // EoS it and increment its playcount etc.
+            if (CurrentState != PlayerState.Loaded && CurrentState != PlayerState.Loading) {
+                // Set the current track as fully played before signaling EndOfStream.
+                ServiceManager.PlayerEngine.IncrementLastPlayed (1.0);
+                OnEventChanged (PlayerEvent.EndOfStream);
+                OnEventChanged (PlayerEvent.StartOfStream);
+            }
         }
 
-        private void HandleError (Enum domain, string error_message, string debug)
+        private void HandleError (GLib.GException ex)
         {
             TrackInfo failed_track = CurrentTrack;
             Close (true);
 
-            error_message = error_message ?? Catalog.GetString ("Unknown Error");
+            var error_message = String.IsNullOrEmpty (ex.Message) ? Catalog.GetString ("Unknown Error") : ex.Message;
 
-            if (domain is ResourceError) {
-                ResourceError domain_code = (ResourceError)domain;
+            if (ex.Domain == Global.ResourceErrorQuark ()) {
+                ResourceError domain_code = (ResourceError)ex.Code;
                 if (failed_track != null) {
                     switch (domain_code) {
                     case ResourceError.NotFound:
@@ -577,8 +623,8 @@ namespace Banshee.GStreamerSharp
                     }
                 }
                 Log.Error (String.Format ("GStreamer resource error: {0}", domain_code), false);
-            } else if (domain is StreamError) {
-                StreamError domain_code = (StreamError)domain;
+            } else if (ex.Domain == Global.StreamErrorQuark ()) {
+                StreamError domain_code = (StreamError)ex.Code;
                 if (failed_track != null) {
                     switch (domain_code) {
                     case StreamError.CodecNotFound:
@@ -590,8 +636,8 @@ namespace Banshee.GStreamerSharp
                 }
 
                 Log.Error (String.Format ("GStreamer stream error: {0}", domain_code), false);
-            } else if (domain is CoreError) {
-                CoreError domain_code = (CoreError)domain;
+            } else if (ex.Domain == Global.CoreErrorQuark ()) {
+                CoreError domain_code = (CoreError)ex.Code;
                 if (failed_track != null) {
                     switch (domain_code) {
                     case CoreError.MissingPlugin:
@@ -603,10 +649,10 @@ namespace Banshee.GStreamerSharp
                 }
 
                 if (domain_code != CoreError.MissingPlugin) {
-                    Log.Error (String.Format ("GStreamer core error: {0}", (CoreError)domain), false);
+                    Log.Error (String.Format ("GStreamer core error: {0}", domain_code), false);
                 }
-            } else if (domain is LibraryError) {
-                Log.Error (String.Format ("GStreamer library error: {0}", (LibraryError)domain), false);
+            } else if (ex.Domain == Global.LibraryErrorQuark ()) {
+                Log.Error (String.Format ("GStreamer library error: {0}", ex.Code), false);
             }
 
             OnEventChanged (new PlayerEventErrorArgs (error_message));
@@ -633,22 +679,12 @@ namespace Banshee.GStreamerSharp
             }
         }
 
-        private void HandleTag (Pad pad, TagList tag_list)
+        private void HandleTag (TagList tag_list, string tagname)
         {
-            foreach (string tag in tag_list.Tags) {
-                if (String.IsNullOrEmpty (tag)) {
-                    continue;
-                }
-
-                if (tag_list.GetTagSize (tag) < 1) {
-                    continue;
-                }
-
-                List tags = tag_list.GetTag (tag);
-
-                foreach (object o in tags) {
-                    OnTagFound (new StreamTag () { Name = tag, Value = o });
-                }
+            for (uint i = 0; i < tag_list.GetTagSize (tagname); i++) {
+                GLib.Value val = tag_list.GetValueIndex (tagname, i);
+                OnTagFound (new StreamTag () { Name = tagname, Value = val.Val });
+                val.Dispose ();
             }
         }
 
@@ -692,7 +728,7 @@ namespace Banshee.GStreamerSharp
                 playbin.SetState (Gst.State.Ready);
             }
 
-            playbin.Uri = uri.AbsoluteUri;
+            playbin ["uri"] = uri.AbsoluteUri;
             if (maybeVideo) {
                 // Lookup for subtitle files with same name/folder
                 LookupForSubtitle (uri);
@@ -705,10 +741,10 @@ namespace Banshee.GStreamerSharp
             SafeUri suburi;
             int dot;
             // Always enable rendering of subtitles
-            int flags;
-            flags = (int)playbin.Flags;
+            uint flags;
+            flags = (uint)playbin["flags"];
             flags |= (1 << 2);//GST_PLAY_FLAG_TEXT
-            playbin.Flags = (ObjectFlags)flags;
+            playbin ["flags"] = flags;
 
             Log.Debug ("[subtitle]: looking for subtitles for video file");
             scheme = uri.Scheme;
@@ -728,7 +764,7 @@ namespace Banshee.GStreamerSharp
                 suburi = new SafeUri (subfile);
                 if (Banshee.IO.File.Exists (suburi)) {
                     Log.DebugFormat ("[subtitle]: Found subtitle file: {0}", subfile);
-                    playbin.Suburi = subfile;
+                    playbin ["suburi"] = subfile;
                     return;
                 }
             }
@@ -737,13 +773,13 @@ namespace Banshee.GStreamerSharp
         public override void Play ()
         {
             playbin.SetState (Gst.State.Playing);
-            OnStateChanged (PlayerState.Playing);
+            video_manager.InvalidateOverlay ();
+            video_manager.MaybePrepareOverlay ();
         }
 
         public override void Pause ()
         {
             playbin.SetState (Gst.State.Paused);
-            OnStateChanged (PlayerState.Paused);
         }
 
         public override void Close (bool fullShutdown)
@@ -754,10 +790,19 @@ namespace Banshee.GStreamerSharp
 
         public override string GetSubtitleDescription (int index)
         {
-            return playbin.GetTextTags (index)
-             .GetTag (Gst.Tag.LanguageCode)
-             .Cast<string> ()
-             .FirstOrDefault (t => t != null);
+            var list = (TagList)playbin.Emit ("get-text-tags", new object[] { index });
+
+            if (list == null)
+                return String.Empty;
+
+            string code;
+            if (!list.GetString (Constants.TAG_LANGUAGE_CODE, out code))
+                return String.Empty;
+
+            var name = Gst.Tags.GlobalTag.TagGetLanguageName (code);
+            Log.Debug ("Subtitle language code " + code + " resolved to " + name);
+
+            return name;
         }
 
         public override ushort Volume {
@@ -768,6 +813,8 @@ namespace Banshee.GStreamerSharp
                 if (audio_sink.VolumeNeedsSaving) {
                     PlayerEngineService.VolumeSchema.Set (value);
                 }
+
+                OnEventChanged (PlayerEvent.Volume);
             }
         }
 
@@ -775,12 +822,28 @@ namespace Banshee.GStreamerSharp
             get { return true; }
         }
 
+        private bool GaplessEnabled {
+            get { return gapless_enabled; }
+            set
+            {
+                if (value == gapless_enabled) {
+                    return;
+                }
+                gapless_enabled = value;
+                if (value) {
+                    playbin.Connect ("about-to-finish", OnAboutToFinish);
+                } else {
+                    playbin.Disconnect ("about-to-finish", OnAboutToFinish);
+                }
+            }
+        }
+
         private static Format query_format = Format.Time;
         public override uint Position {
             get {
                 long pos;
-                playbin.QueryPosition (ref query_format, out pos);
-                return (uint) ((ulong)pos / Gst.Clock.MSecond);
+                playbin.QueryPosition (query_format, out pos);
+                return (uint) ((ulong)pos / Constants.MSECOND);
             }
             set { Seek (value); }
         }
@@ -788,8 +851,8 @@ namespace Banshee.GStreamerSharp
         public override uint Length {
             get {
                 long duration;
-                playbin.QueryDuration (ref query_format, out duration);
-                return (uint) ((ulong)duration / Gst.Clock.MSecond);
+                playbin.QueryDuration (query_format, out duration);
+                return (uint) ((ulong)duration / Constants.MSECOND);
             }
         }
 
@@ -860,7 +923,7 @@ namespace Banshee.GStreamerSharp
         }
 
         public override int SubtitleCount {
-            get { return playbin.NText; }
+            get { return (int)playbin ["n-text"]; }
         }
 
         public override int SubtitleIndex {
@@ -871,11 +934,11 @@ namespace Banshee.GStreamerSharp
 
                 if (value == -1) {
                     flags &= ~(1 << 2);//GST_PLAY_FLAG_TEXT
-                    playbin.Flags = (ObjectFlags)flags;
+                    playbin ["flags"] = flags;
                 } else {
                     flags |= (1 << 2);//GST_PLAY_FLAG_TEXT
-                    playbin.Flags = (ObjectFlags)flags;
-                    playbin.CurrentText = value;
+                    playbin ["flags"] = flags;
+                    playbin ["current-text"] = value;
                 }
             }
         }
@@ -883,32 +946,32 @@ namespace Banshee.GStreamerSharp
         public override SafeUri SubtitleUri {
             set {
                 long pos = -1;
-                State state;
+                State state, pending;
                 Format format = Format.Bytes;
                 bool paused = false;
 
                 // GStreamer playbin does not support setting the suburi during playback
                 // so we have to stop/play and seek
-                playbin.GetState (out state, 0);
+                playbin.GetState (out state, out pending, 0);
                 paused = (state == State.Paused);
                 if (state >= State.Paused) {
-                    playbin.QueryPosition (ref format, out pos);
+                    playbin.QueryPosition (format, out pos);
                     playbin.SetState (State.Ready);
                     // Wait for the state change to complete
-                    playbin.GetState (out state, 0);
+                    playbin.GetState (out state, out pending, 0);
                 }
 
-                playbin.Suburi = value.AbsoluteUri;
+                playbin ["suburi"] = value.AbsoluteUri;
                 playbin.SetState (paused ? State.Paused : State.Playing);
 
                 // Wait for the state change to complete
-                playbin.GetState (out state, 0);
+                playbin.GetState (out state, out pending, 0);
 
                 if (pos != -1) {
-                    playbin.Seek (format, SeekFlags.Flush | SeekFlags.KeyUnit, pos);
+                    playbin.SeekSimple (format, SeekFlags.Flush | SeekFlags.KeyUnit, pos);
                 }
             }
-            get { return new SafeUri (playbin.Suburi); }
+            get { return new SafeUri ((string)playbin ["suburi"]); }
         }
 
 #region DVD support
@@ -977,6 +1040,7 @@ namespace Banshee.GStreamerSharp
 #region Preferences
 
         private PreferenceBase replaygain_preference;
+        private PreferenceBase gapless_preference;
 
         private void InstallPreferences ()
         {
@@ -990,6 +1054,11 @@ namespace Banshee.GStreamerSharp
                 Catalog.GetString ("For tracks that have ReplayGain data, automatically scale (normalize) playback volume"),
                 delegate { audio_sink.ReplayGainEnabled = ReplayGainEnabledSchema.Get (); }
             ));
+            gapless_preference = service["general"]["misc"].Add (new SchemaPreference<bool> (GaplessEnabledSchema,
+                Catalog.GetString ("Enable _gapless playback"),
+                Catalog.GetString ("Eliminate the small playback gap on track change. Useful for concept albums and classical music"),
+                () => { GaplessEnabled = GaplessEnabledSchema.Get (); }
+            ));
         }
 
         private void UninstallPreferences ()
@@ -1010,6 +1079,13 @@ namespace Banshee.GStreamerSharp
             "If ReplayGain data is present on tracks when playing, allow volume scaling"
         );
 
+        public static readonly SchemaEntry<bool> GaplessEnabledSchema = new SchemaEntry<bool> (
+            "player_engine", "gapless_playback_enabled",
+            true,
+            "Enable gapless playback",
+            "Eliminate the small playback gap on track change. Useful for concept albums and classical music"
+        );
+
 #endregion
     }
 }
