@@ -34,13 +34,10 @@ using System.IO;
 using Mono.Unix;
 
 using Hyena;
-using Hyena.Data.Sqlite;
 
-using Banshee.Base;
 using Banshee.Sources;
 using Banshee.Collection;
 using Banshee.Collection.Database;
-using Banshee.Configuration.Schema;
 using Banshee.ServiceStack;
 using Banshee.Streaming;
 
@@ -168,6 +165,12 @@ namespace Banshee.Collection.Database
                 return null;
             }
 
+            if (!Banshee.IO.File.GetPermissions (uri).IsReadable) {
+                throw new InvalidFileException (String.Format (
+                    Catalog.GetString ("File is not readable so it could not be imported: {0}"),
+                    Path.GetFileName (uri.LocalPath)));
+            }
+
             if (Banshee.IO.File.GetSize (uri) == 0) {
                 throw new InvalidFileException (String.Format (
                     Catalog.GetString ("File is empty so it could not be imported: {0}"),
@@ -251,7 +254,8 @@ namespace Banshee.Collection.Database
 
         public void NotifyAllSources ()
         {
-            foreach (int primary_source_id in counts.Keys) {
+            var all_primary_source_ids = new List<long> (counts.Keys);
+            foreach (long primary_source_id in all_primary_source_ids) {
                 PrimarySource.GetById (primary_source_id).NotifyTracksAdded ();
             }
             counts.Clear ();
