@@ -3,8 +3,10 @@
 //
 // Author:
 //   Gabriel Burt <gburt@novell.com>
+//   Andrés G. Aragoneses <knocte@gmail.com>
 //
 // Copyright (C) 2010 Novell, Inc.
+// Copyright (C) 2014 Andrés G. Aragoneses
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -29,6 +31,7 @@
 #if ENABLE_TESTS
 
 using System;
+using System.Collections.Generic;
 using System.Linq;
 using System.Threading;
 
@@ -91,8 +94,71 @@ namespace Banshee.MediaEngine
             AssertTransition (() => service.TrackInfoUpdated (), PlayerEvent.TrackInfoUpdated);
             LoadAndPlay ("A_girl.ogg");
             AssertTransition (() => service.TrackInfoUpdated (), PlayerEvent.TrackInfoUpdated);
+        }
 
-            AssertTransition (() => service.Dispose (), PlayerState.Paused, PlayerState.Idle);
+        [Test] // https://bugzilla.gnome.org/show_bug.cgi?id=722731
+        public void TestThatInvalidTracksShouldNotBecomeCurrentTrack ()
+        {
+            var a_valid_track = "A_boy.ogg";
+            var a_valid_uri = new SafeUri (Paths.Combine (TestsDir, "data", a_valid_track));
+
+            var a_track_that_doesnt_exit = "this_does_not_exist_in_the_data_folder.ogg";
+            var an_invalid_track = new SafeUri (Paths.Combine (TestsDir, "data", a_track_that_doesnt_exit));
+
+            TrackInfo current_track = null;
+
+            WaitUntil (PlayerEvent.Error, () => {
+                service.Open (a_valid_uri);
+                current_track = service.CurrentTrack;
+                Assert.IsNotNull (current_track);
+                Assert.IsTrue (current_track.Uri.AbsolutePath.EndsWith (a_valid_track));
+
+                service.SetNextTrack (an_invalid_track);
+                service.Play ();
+            });
+
+            WaitUntil (PlayerEvent.StartOfStream, () => {
+                service.Open (a_valid_uri);
+                service.Play ();
+            });
+
+            current_track = service.CurrentTrack;
+            Assert.IsNotNull (current_track);
+            var actual_filename = System.IO.Path.GetFileName (SafeUri.UriToFilename (current_track.Uri));
+            Assert.AreEqual (a_valid_track, actual_filename);
+        }
+
+        private void StopEngine ()
+        {
+            service.SetNextTrack ((TrackInfo)null);
+            service.Play ();
+        }
+
+        [Test]
+        // make sure https://bugzilla.gnome.org/show_bug.cgi?id=722769 doesn't happen, or that our workaround works
+        public void TestThatRequestNextTrackEventsAreNotDuplicated ()
+        {
+            WaitUntil (PlayerState.Idle, StopEngine);
+
+            var a_valid_track = "A_boy.ogg";
+            var a_valid_uri = new SafeUri (Paths.Combine (TestsDir, "data", a_valid_track));
+
+            var ignore_all_except_reqnext_and_idle = new Func<PlayerState?, PlayerEvent?, bool> ((s, e) => {
+                bool is_req_or_idle = PlayerEvent.RequestNextTrack.Equals (e.Value) ||
+                    (PlayerEvent.StateChange.Equals (e.Value) && PlayerState.Idle.Equals (s.Value));
+                return !is_req_or_idle;
+            });
+
+            System.Action action = () => {
+                service.Open (a_valid_uri);
+                service.Play ();
+            };
+            WaitUntil (PlayerState.Playing,
+                action);
+
+            WaitFor (ignore_all_except_reqnext_and_idle,
+                PlayerEvent.RequestNextTrack, // <- this is the key, notice that we expect only one, not two
+                PlayerState.Idle);
         }
 
         private void LoadAndPlay (string filename)
@@ -149,9 +215,14 @@ namespace Banshee.MediaEngine
             WaitFor (default_ignore, states);
         }
 
+        private void WaitFor (System.Func<PlayerState?, PlayerEvent?, bool> ignore, System.Action action, params object [] states)
+        {
+            AssertTransition (ignore, action, states);
+        }
+
         private void WaitFor (System.Func<PlayerState?, PlayerEvent?, bool> ignore, params object [] states)
         {
-            AssertTransition (ignore, null, states);
+            WaitFor (ignore, null, states);
         }
 
         private void AssertTransition (System.Action action, params object [] states)
@@ -167,9 +238,19 @@ namespace Banshee.MediaEngine
         private void AssertTransition (System.Func<PlayerState?, PlayerEvent?, bool> ignore, System.Action action, params object [] states)
         {
             Log.DebugFormat ("AssertTransition: {0}", String.Join (", ", states.Select (s => s.ToString ()).ToArray ()));
+
+            foreach (var state in states) {
+                if (state == null) {
+                    throw new ArgumentException ("states array cannot contain nulls");
+                }
+                if (state.GetType () != typeof (PlayerEvent) && state.GetType () != typeof (PlayerState)) {
+                    throw new ArgumentException ("states array can only contain states or events, not " + state.GetType ().FullName);
+                }
+            }
+
             int result_count = 0;
             var reset_event = new ManualResetEvent (false);
-            var handler = new PlayerEventHandler (a => {
+            var handler = new Action<PlayerEventArgs> (a => {
                 lock (states) {
                     if (result_count < states.Length) {
                         var sca = a as PlayerEventStateChangeArgs;
@@ -191,29 +272,49 @@ namespace Banshee.MediaEngine
                         var evnt = (states[result_count] as PlayerEvent?) ?? PlayerEvent.StateChange;
                         var state = states[result_count] as PlayerState?;
 
-                        result_count++;
-                        Assert.AreEqual (evnt, last_event);
+                        var evnt_or_state_desc = (evnt == PlayerEvent.StateChange ?
+                            evnt.ToString() + ">" + states[result_count].ToString () : evnt.ToString ());
+                        var last_evnt_or_state_desc = (last_event == PlayerEvent.StateChange ?
+                            last_event.ToString () + ">" + sca.Current.ToString () : last_event.ToString ());
+
+                        Assert.That (last_event, Is.EqualTo (evnt),
+                            String.Format ("Event[{0}] doesn't match (should be {1}, it was {2})",
+                                           result_count, evnt_or_state_desc, last_evnt_or_state_desc));
                         if (state != null) {
-                            Assert.AreEqual (state, last_state);
+                            Assert.That (last_state, Is.EqualTo (state), String.Format ("State[{0}] doesn't match", result_count));
                         }
+
+                        result_count++;
                     }
                 }
                 reset_event.Set ();
             });
 
-            service.ConnectEvent (handler);
+            AssertionException exception = null;
+            var guarded_handler = new PlayerEventHandler (args => {
+                try {
+                    handler (args);
+                } catch (AssertionException ae) {
+                    exception = ae;
+                }
+            });
+            service.ConnectEvent (guarded_handler);
 
             if (action != null) action ();
 
+            const int seconds = 3;
             while (result_count < states.Length) {
                 reset_event.Reset ();
-                if (!reset_event.WaitOne (3000)) {
-                    Assert.Fail (String.Format ("Waited 3s for state/event, didnt' happen"));
+                if (!reset_event.WaitOne (TimeSpan.FromSeconds (seconds))) {
+                    Assert.Fail (String.Format ("Waited {0}s for state/event, didn't happen", seconds));
                     break;
                 }
+                if (exception != null) {
+                    throw new AssertionException (exception.Message, exception);
+                }
             }
 
-            service.DisconnectEvent (handler);
+            service.DisconnectEvent (guarded_handler);
         }
 
         //[Test]
@@ -279,6 +380,107 @@ namespace Banshee.MediaEngine
         public void ModifyEvent (PlayerEvent eventMask, PlayerEventHandler handler)
         */
 
+        private void WaitUntil (PlayerEvent @event, System.Action action)
+        {
+            WaitUntil (@event, null, action);
+        }
+
+        private void WaitUntil (PlayerState state, System.Action action)
+        {
+            WaitUntil (null, state, action);
+        }
+
+        private void WaitUntil (PlayerEvent? @event, PlayerState? state, System.Action action)
+        {
+            if (action == null) {
+                throw new ArgumentNullException ("action");
+            }
+            if (@event == null && state == null) {
+                throw new ArgumentException ("Event or state must be non-null");
+            }
+            if (@event != null && state != null) {
+                throw new ArgumentException ("Event and state cannot be both non-null");
+            }
+
+            object lock_object = new object ();
+            string evnt_or_state_desc = @event != null ? @event.Value.ToString () : state.Value.ToString ();
+
+            Exception exception = null;
+
+
+            Func<PlayerEventArgs, bool> matches = (a) => {
+                if (a == null) {
+                    throw new ArgumentNullException ("a");
+                }
+                var sca = a as PlayerEventStateChangeArgs;
+                PlayerEvent? last_event = a.Event;
+                PlayerState? last_state = null;
+                if (sca != null) {
+                    last_state = sca.Current;
+                }
+
+                return (@event != null && @event.Value.Equals (last_event.Value))
+                || (state != null && last_state != null && state.Value.Equals (last_state.Value));
+            };
+            var args_queue = new Queue<PlayerEventArgs> ();
+
+            var reset_event = new ManualResetEvent (false);
+            reset_event.Reset ();
+
+            var handler = new PlayerEventHandler (a => {
+                try {
+                    lock (lock_object) {
+                        args_queue.Enqueue (a);
+                    }
+                } catch (Exception ex) {
+                    exception = ex;
+                }
+                reset_event.Set ();
+            });
+            service.ConnectEvent (handler);
+
+            bool return_early = (state != null && service.CurrentState.Equals (state.Value));
+            action ();
+            if (return_early) {
+                return;
+            }
+
+            const int seconds = 3;
+            int count = 0;
+            const int max_count = 10;
+
+            string event_or_state_desc = @event != null ? @event.ToString () : state.ToString ();
+
+            bool found = false;
+            do {
+                if (!reset_event.WaitOne (TimeSpan.FromSeconds (seconds))) {
+                    Assert.Fail (String.Format ("Waited {0}s for {1} at iteration {2}, but didn't happen",
+                                                seconds, evnt_or_state_desc, count));
+                    break;
+                }
+                lock (lock_object) {
+                    while (args_queue.Count > 0) {
+                        var arg = args_queue.Dequeue ();
+                        if (matches (arg)) {
+                            found = true;
+                            break;
+                        }
+                        count++;
+                    }
+                }
+
+                if (exception != null) {
+                    throw exception;
+                }
+                if (count > max_count) {
+                    Assert.Fail (String.Format ("More than {0} events/states happened, but not {1}", max_count, event_or_state_desc));
+                }
+            } while (!found);
+
+            service.DisconnectEvent (handler);
+        }
+
+
         Thread main_thread;
         GLib.MainLoop main_loop;
         bool started;
@@ -308,9 +510,26 @@ namespace Banshee.MediaEngine
         [TestFixtureTearDown]
         public void Teardown ()
         {
-            GLib.Idle.Add (delegate { main_loop.Quit (); return false; });
-            main_thread.Join ();
-            main_thread = null;
+            try {
+
+                var a_valid_track1 = "A_boy.ogg";
+                var a_valid_uri = new SafeUri (Paths.Combine (TestsDir, "data", a_valid_track1));
+                WaitUntil (PlayerState.Playing, () => {
+                    service.Open (a_valid_uri);
+                    service.Play ();
+                });
+
+                AssertTransition (service.Dispose, PlayerState.Paused, PlayerState.Idle);
+
+            } finally {
+
+                GLib.Idle.Add (delegate {
+                    main_loop.Quit ();
+                    return false;
+                });
+                main_thread.Join ();
+                main_thread = null;
+            }
         }
 
         int play_when_idles = 0;
