@@ -119,14 +119,14 @@ namespace Banshee.Sources
             get { return expanded_schema; }
         }
 
-        private int dbid;
-        public int DbId {
+        private long dbid;
+        public long DbId {
             get {
                 if (dbid > 0) {
                     return dbid;
                 }
 
-                dbid = ServiceManager.DbConnection.Query<int> ("SELECT PrimarySourceID FROM CorePrimarySources WHERE StringID = ?", UniqueId);
+                dbid = ServiceManager.DbConnection.Query<long> ("SELECT PrimarySourceID FROM CorePrimarySources WHERE StringID = ?", UniqueId);
                 if (dbid == 0) {
                     dbid = ServiceManager.DbConnection.Execute ("INSERT INTO CorePrimarySources (StringID, IsTemporary) VALUES (?, ?)", UniqueId, IsTemporary);
                 } else {
@@ -190,8 +190,8 @@ namespace Banshee.Sources
         public event TrackEventHandler TracksChanged;
         public event TrackEventHandler TracksDeleted;
 
-        private static Dictionary<int, PrimarySource> primary_sources = new Dictionary<int, PrimarySource> ();
-        public static PrimarySource GetById (int id)
+        private static Dictionary<long, PrimarySource> primary_sources = new Dictionary<long, PrimarySource> ();
+        public static PrimarySource GetById (long id)
         {
             return (primary_sources.ContainsKey (id)) ? primary_sources[id] : null;
         }
@@ -510,8 +510,8 @@ namespace Banshee.Sources
         protected virtual void DeleteTrackList (CachedList<DatabaseTrackInfo> list)
         {
             is_deleting = true;
-            DeleteTrackJob.Total += (int) list.Count;
-            List<DatabaseTrackInfo> skip_deletion = null;
+            DeleteTrackJob.Total += list.Count;
+            var skip_deletion = new List<DatabaseTrackInfo> ();
 
             // Remove from file system
             foreach (DatabaseTrackInfo track in list) {
@@ -520,12 +520,14 @@ namespace Banshee.Sources
                     continue;
                 }
 
+                if (DeleteTrackJob.IsCancelRequested) {
+                    skip_deletion.Add (track);
+                    continue;
+                }
+
                 try {
                     DeleteTrackJob.Status = String.Format ("{0} - {1}", track.ArtistName, track.TrackTitle);
                     if (!DeleteTrack (track)) {
-                        if (skip_deletion == null) {
-                            skip_deletion = new List<DatabaseTrackInfo> ();
-                        }
                         skip_deletion.Add (track);
                     }
                 } catch (Exception e) {
@@ -539,17 +541,15 @@ namespace Banshee.Sources
                 }
             }
 
+            if (!DeleteTrackJob.IsFinished || DeleteTrackJob.IsCancelRequested) {
+                delete_track_job.Finish ();
+            }
+            delete_track_job = null;
             is_deleting = false;
 
-            if (DeleteTrackJob.Total == DeleteTrackJob.Completed) {
-                delete_track_job.Finish ();
-                delete_track_job = null;
-            }
-
-            if (skip_deletion != null) {
+            if (skip_deletion.Count > 0) {
                 list.Remove (skip_deletion);
                 skip_deletion.Clear ();
-                skip_deletion = null;
             }
 
             // Remove from database
@@ -602,7 +602,7 @@ namespace Banshee.Sources
             return true;
         }
 
-        public int GetTrackIdForUri (string uri)
+        public long GetTrackIdForUri (string uri)
         {
             return DatabaseTrackInfo.GetTrackIdForUri (new SafeUri (uri), DbId);
         }
@@ -628,7 +628,7 @@ namespace Banshee.Sources
         {
             CachedList<DatabaseTrackInfo> list = cached_list as CachedList<DatabaseTrackInfo>;
             is_adding = true;
-            AddTrackJob.Total += (int) list.Count;
+            AddTrackJob.Total += list.Count;
 
             foreach (DatabaseTrackInfo track in list) {
                 if (AddTrackJob.IsCancelRequested) {
@@ -727,6 +727,7 @@ namespace Banshee.Sources
                         delete_track_job.SetResources (Resource.Cpu, Resource.Database);
                         delete_track_job.PriorityHints = PriorityHints.SpeedSensitive | PriorityHints.DataLossIfStopped;
                         delete_track_job.DelayShow = DelayDeleteJob;
+                        delete_track_job.CanCancel = true;
                         delete_track_job.Register ();
                     }
                 }
