@@ -83,7 +83,7 @@ namespace Banshee.Gui.Dialogs
             // than plain Name, since it's been depreciated. We can't do that
             // yet though since it breaks stuff for other people.
             Name = "Banshee";
-            Logo = Gdk.Pixbuf.LoadFromResource (ApplicationContext.Debugging ? /*"jcastro.png"*/ "banshee-logo.png" : "banshee-logo.png");
+            Logo = Gdk.Pixbuf.LoadFromResource ("banshee-logo.png");
             Version = Banshee.ServiceStack.Application.DisplayVersion == Banshee.ServiceStack.Application.Version
                 ? Banshee.ServiceStack.Application.DisplayVersion
                 : String.Format ("{0} ({1})",
@@ -94,7 +94,7 @@ namespace Banshee.Gui.Dialogs
                 "Copyright \u00a9 2005\u2013{0} Novell, Inc.\n" +
                 "Copyright \u00a9 2005\u2013{1} Others\n" +
                 "Copyright \u00a9 2005 Aaron Bockover"
-            ), "2011", "2012");
+            ), "2011", "2014");
 
             Website = "http://banshee.fm/";
             WebsiteLabel = Catalog.GetString ("Banshee Website");
