@@ -39,7 +39,6 @@ using Banshee.Base;
 using Banshee.Hardware;
 using Banshee.Collection;
 using Banshee.Collection.Database;
-using Folder = Banshee.IO.Directory;
 
 namespace Banshee.Dap.MassStorage
 {
@@ -117,18 +116,28 @@ namespace Banshee.Dap.MassStorage
             cover_art_file_type = GetPreferredValue ("cover_art_file_type", config, DefaultCoverArtFileType);
             cover_art_file_name = GetPreferredValue ("cover_art_file_name", config, DefaultCoverArtFileName);
             cover_art_size = GetPreferredValue ("cover_art_size", config, DefaultCoverArtSize);
-            audio_folders = MergeValues ("audio_folders", config, DefaultAudioFolders);
-            video_folders = MergeValues ("video_folders", config, DefaultVideoFolders);
+
+            var all_audio_folders = MergeValues ("audio_folders", config, DefaultAudioFolders);
+            var all_video_folders = MergeValues ("video_folders", config, DefaultVideoFolders);
+            if (!FileSystem.IsCaseSensitive (source.Volume)) {
+                all_audio_folders = all_audio_folders.Distinct (StringComparer.CurrentCultureIgnoreCase).ToArray ();
+                all_video_folders = all_video_folders.Distinct (StringComparer.CurrentCultureIgnoreCase).ToArray ();
+            }
+            audio_folders = all_audio_folders;
+            video_folders = all_video_folders;
+
             playback_mime_types = MergeValues ("output_formats", config, DefaultPlaybackMimeTypes);
             playlist_formats = MergeValues ("playlist_formats", config, DefaultPlaylistFormats);
-            playlist_paths = new string [] { GetPreferredValue ("playlist_path", config, DefaultPlaylistPath) };
+            var playlist_path = GetPreferredValue ("playlist_path", config, DefaultPlaylistPath);
+            PlaylistPaths = playlist_path != null ? new string [] { playlist_path } : new string [0];
+
             folder_depth = GetPreferredValue ("folder_depth", config, DefaultFolderDepth);
 
             string preferred_folder_separator = GetPreferredValue ("folder_separator", config, DefaultFolderSeparator);
-            if (preferred_folder_separator == Folder.DosSeparator.ToString () || preferred_folder_separator == "DOS") {
-                folder_separator = Folder.DosSeparator;
+            if (preferred_folder_separator == Paths.Folder.DosSeparator.ToString () || preferred_folder_separator == "DOS") {
+                folder_separator = Paths.Folder.DosSeparator;
             } else {
-                folder_separator = Folder.UnixSeparator;
+                folder_separator = Paths.Folder.UnixSeparator;
             }
         }
 
@@ -208,6 +217,10 @@ namespace Banshee.Dap.MassStorage
             get { return FolderDepth; }
         }
 
+        internal virtual Uri RootPath {
+            get { return null; }
+        }
+
         protected virtual string DefaultFolderSeparator {
             get { return null; }
         }
@@ -266,9 +279,8 @@ namespace Banshee.Dap.MassStorage
             get { return null; }
         }
 
-        private string [] playlist_paths;
         public virtual string [] PlaylistPaths {
-            get { return playlist_paths; }
+            get; private set;
         }
 
         protected virtual string[] DefaultPlaybackMimeTypes {
