@@ -32,14 +32,11 @@ using System.Threading;
 using Mono.Unix;
 
 using Hyena;
-using Hyena.Collections;
 using Mtp;
 using MTP = Mtp;
 
-using Banshee.Base;
 using Banshee.Dap;
 using Banshee.ServiceStack;
-using Banshee.Library;
 using Banshee.Sources;
 using Banshee.Playlist;
 using Banshee.Configuration;
@@ -51,7 +48,12 @@ namespace Banshee.Dap.Mtp
 {
     public class MtpSource : DapSource
     {
-		private MtpDevice mtp_device;
+
+        protected override object InternalLock {
+            get { return mtp_device; }
+        }
+        private MtpDevice mtp_device;
+
         //private bool supports_jpegs = false;
         private Dictionary<long, Track> track_map;
 
@@ -273,7 +275,7 @@ namespace Banshee.Dap.Mtp
                     using (System.IO.TextWriter writer = new System.IO.StreamWriter (Banshee.IO.File.OpenWrite (empty_file, true))) {
                         writer.Write ("foo");
                     }
-                    Track mtp_track = new Track (System.IO.Path.GetFileName (empty_file.LocalPath), 3);
+                    Track mtp_track = new Track (System.IO.Path.GetFileName (empty_file.LocalPath), 3, mtp_device);
 
                     mtp_device.UploadTrack (empty_file.AbsolutePath, mtp_track, mtp_device.MusicFolder);
                     mtp_device.Remove (mtp_track);
@@ -311,7 +313,7 @@ namespace Banshee.Dap.Mtp
         private long bytes_capacity;
         public override long BytesCapacity {
             get {
-                if (Monitor.TryEnter (mtp_device)) {
+                if (mtp_device != null && Monitor.TryEnter (mtp_device)) {
                     try {
                         bytes_capacity = 0;
                         foreach (DeviceStorage s in mtp_device.GetStorage ()) {
@@ -425,7 +427,7 @@ namespace Banshee.Dap.Mtp
 
         public Track TrackInfoToMtpTrack (TrackInfo track, SafeUri fromUri)
         {
-			Track f = new Track (System.IO.Path.GetFileName (fromUri.LocalPath), (ulong) Banshee.IO.File.GetSize (fromUri));
+            Track f = new Track (System.IO.Path.GetFileName (fromUri.LocalPath), (ulong) Banshee.IO.File.GetSize (fromUri), mtp_device);
             MtpTrackInfo.ToMtpTrack (track, f);
             return f;
         }
