@@ -28,7 +28,6 @@
 //
 
 using System;
-using System.Linq;
 using System.IO;
 using System.Collections.Generic;
 using System.Threading;
@@ -37,7 +36,6 @@ using Mono.Unix;
 using Hyena;
 using Hyena.Data.Sqlite;
 
-using Banshee.Base;
 using Banshee.ServiceStack;
 using Banshee.Sources;
 using Banshee.Collection;
@@ -96,15 +94,30 @@ namespace Banshee.Dap
             TypeUniqueId = device.Serial;
         }
 
+        private object internal_lock = new object ();
+        protected virtual object InternalLock {
+            get { return internal_lock; }
+        }
+
         public override void Dispose ()
         {
+            lock (InternalLock) {
+                if (load_thread != null) {
+                    if (load_thread.IsAlive) {
+                        load_thread.Abort ();
+                    }
+                    load_thread = null;
+                }
+            }
+
             Flush ();
 
             PurgeTemporaryPlaylists ();
             PurgeTracks ();
 
             if (dap_info_bar != null) {
-                dap_info_bar.Destroy ();
+                var info_bar = dap_info_bar;
+                ThreadAssist.ProxyToMain (info_bar.Destroy);
                 dap_info_bar = null;
             }
 
@@ -282,10 +295,12 @@ namespace Banshee.Dap
 
         public void LoadDeviceContents ()
         {
-            ThreadPool.QueueUserWorkItem (ThreadedLoadDeviceContents);
+            load_thread = ThreadAssist.Spawn (ThreadedLoadDeviceContents);
         }
 
-        private void ThreadedLoadDeviceContents (object state)
+        private Thread load_thread;
+
+        private void ThreadedLoadDeviceContents ()
         {
             try {
                 PurgeTracks ();
@@ -531,18 +546,6 @@ namespace Banshee.Dap
             get { return BytesCapacity - BytesUsed - Math.Max (0, BytesReserved - BytesData); }
         }
 
-        public override bool CanRemoveTracks {
-            get { return base.CanRemoveTracks; }
-        }
-
-        public override bool CanDeleteTracks {
-            get { return base.CanDeleteTracks; }
-        }
-
-        public override bool CanAddTracks {
-            get { return base.CanAddTracks; }
-        }
-
         public override bool PlaylistsReadOnly {
             get { return IsReadOnly; }
         }
