@@ -5,8 +5,11 @@
 //   Chris Toshok <toshok@ximian.com>
 //   Alexander Hixon <hixon.alexander@mediati.org>
 //   Phil Trimble <philtrimble@gmail.com>
+//   Andres G. Aragoneses <knocte@gmail.com>
 //
 // Copyright (C) 2005-2008 Novell, Inc.
+// Copyright (C) 2013 Phil Trimble
+// Copyright (C) 2013 Andres G. Aragoneses
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -30,6 +33,7 @@
 
 using System;
 using System.Collections.Generic;
+using System.Linq;
 using System.IO;
 using System.Net;
 using System.Text;
@@ -40,9 +44,30 @@ using System.Web;
 using Hyena;
 using Hyena.Json;
 using Mono.Unix;
+using System.Collections.Specialized;
 
 namespace Lastfm
 {
+    public class SubmissionStartEventArgs : EventArgs
+    {
+        public SubmissionStartEventArgs (int total_count)
+        {
+            TotalCount = total_count;
+        }
+
+        public int TotalCount { get; private set; }
+    }
+
+    public class SubmissionUpdateEventArgs : EventArgs
+    {
+        public SubmissionUpdateEventArgs (int update_count)
+        {
+            UpdateCount = update_count;
+        }
+
+        public int UpdateCount { get; private set; }
+    }
+
     public class AudioscrobblerConnection
     {
         private enum State {
@@ -66,6 +91,10 @@ namespace Lastfm
             get { return started; }
         }
 
+        public event EventHandler<SubmissionStartEventArgs> SubmissionStart;
+        public event EventHandler<SubmissionUpdateEventArgs> SubmissionUpdate;
+        public event EventHandler SubmissionEnd;
+
         private System.Timers.Timer timer;
         private DateTime next_interval;
 
@@ -164,13 +193,15 @@ namespace Lastfm
             // and address changes in our engine state
             switch (state) {
             case State.Idle:
-                if (queue.Count > 0) {
+                if (queue.Any ()) {
                     state = State.NeedTransmit;
+                    RaiseSubmissionStart (queue.Count);
                 } else if (current_now_playing_request != null) {
                     // Now playing info needs to be sent
                     NowPlaying (current_now_playing_request);
                 } else {
                     StopTransitionHandler ();
+                    RaiseSubmissionEnd ();
                 }
 
                 break;
@@ -199,33 +230,30 @@ namespace Lastfm
                 return;
             }
 
+            state = State.Transmitting;
             current_scrobble_request = new LastfmRequest ("track.scrobble", RequestType.Write, ResponseFormat.Json);
-            IList<IQueuedTrack> tracks = queue.GetTracks ();
 
-             for (int i = 0; i < tracks.Count; i++) {
-                IQueuedTrack track = tracks[i];
+            int trackCount = 0;
+            while (true) {
+                IQueuedTrack track = queue.GetNextTrack ();
+                if (track == null ||
+                    // Last.fm can technically handle up to 50 songs in one request
+                    // but let's not use the top limit
+                    trackCount == 40) {
+                    break;
+                 }
 
-                string str_track_number = String.Empty;
-                if (track.TrackNumber != 0) {
-                    str_track_number = track.TrackNumber.ToString();
+                try {
+                    current_scrobble_request.AddParameters (GetTrackParameters (track, trackCount));
+                    trackCount++;
+                } catch (MaxSizeExceededException) {
+                    break;
                 }
-
-                bool chosen_by_user = (track.TrackAuth.Length == 0);
-
-                current_scrobble_request.AddParameter (String.Format ("timestamp[{0}]", i), track.StartTime.ToString ());
-                current_scrobble_request.AddParameter (String.Format ("track[{0}]", i), track.Title);
-                current_scrobble_request.AddParameter (String.Format ("artist[{0}]", i), track.Artist);
-                current_scrobble_request.AddParameter (String.Format ("album[{0}]", i), track.Album);
-                current_scrobble_request.AddParameter (String.Format ("trackNumber[{0}]", i), str_track_number);
-                current_scrobble_request.AddParameter (String.Format ("duration[{0}]", i), track.Duration.ToString ());
-                current_scrobble_request.AddParameter (String.Format ("mbid[{0}]", i), track.MusicBrainzId);
-                current_scrobble_request.AddParameter (String.Format ("chosenByUser[{0}]", i), chosen_by_user ? "1" : "0");
             }
 
             Log.DebugFormat ("Last.fm scrobbler sending '{0}'", current_scrobble_request.ToString ());
 
-            state = State.Transmitting;
-            current_async_result = current_scrobble_request.BeginSend (OnScrobbleResponse, tracks.Count);
+            current_async_result = current_scrobble_request.BeginSend (OnScrobbleResponse, trackCount);
             state = State.WaitingForResponse;
             if (!(current_async_result.AsyncWaitHandle.WaitOne (TIME_OUT, false))) {
                 Log.Warning ("Audioscrobbler upload failed", "The request timed out and was aborted", false);
@@ -235,6 +263,26 @@ namespace Lastfm
             }
         }
 
+        private static NameValueCollection GetTrackParameters (IQueuedTrack track, int trackCount)
+        {
+            string str_track_number = String.Empty;
+            if (track.TrackNumber != 0) {
+                str_track_number = track.TrackNumber.ToString();
+            }
+            bool chosen_by_user = (track.TrackAuth.Length == 0);
+
+            return new NameValueCollection () {
+                { String.Format ("timestamp[{0}]", trackCount), track.StartTime.ToString () },
+                { String.Format ("track[{0}]", trackCount), track.Title },
+                { String.Format ("artist[{0}]", trackCount), track.Artist },
+                { String.Format ("album[{0}]", trackCount), track.Album },
+                { String.Format ("trackNumber[{0}]", trackCount), str_track_number },
+                { String.Format ("duration[{0}]", trackCount), track.Duration.ToString () },
+                { String.Format ("mbid[{0}]", trackCount), track.MusicBrainzId },
+                { String.Format ("chosenByUser[{0}]", trackCount), chosen_by_user ? "1" : "0" }
+            };
+        }
+
         private void OnScrobbleResponse (IAsyncResult ar)
         {
             int nb_tracks_scrobbled = 0;
@@ -248,27 +296,25 @@ namespace Lastfm
                 return;
             }
 
-            var response = current_scrobble_request.GetResponseObject ();
+            JsonObject response = null;
+            try {
+                response = current_scrobble_request.GetResponseObject ();
+            } catch (Exception e) {
+                Log.Exception ("Failed to process the scrobble response", e);
+                state = State.Idle;
+                return;
+            }
+
             var error = current_scrobble_request.GetError ();
             if (error == StationError.ServiceOffline || error == StationError.TemporarilyUnavailable) {
                 Log.WarningFormat ("Lastfm is temporarily unavailable: {0}", (string)response ["message"]);
                 next_interval = DateTime.Now + new TimeSpan (0, 0, RETRY_SECONDS);
                 hard_failures++;
                 state = State.Idle;
-            } else if (error != StationError.None) {
-                // TODO: If error == StationError.InvalidSessionKey,
-                // suggest to the user to (re)do the Last.fm authentication.
-                hard_failures++;
+                return;
+            }
 
-                queue.RemoveInvalidTracks ();
-
-                // if there are still valid tracks in the queue then retransmit on the next interval
-                if (queue.Count > 0) {
-                    state = State.NeedTransmit;
-                } else {
-                    state = State.Idle;
-                }
-            } else {
+            if (error == StationError.None) {
                 try {
                     var scrobbles = (JsonObject)response["scrobbles"];
                     var scrobbles_attr = (JsonObject)scrobbles["@attr"];
@@ -294,9 +340,17 @@ namespace Lastfm
                 // we succeeded, pop the elements off our queue
                 queue.RemoveRange (0, nb_tracks_scrobbled);
                 queue.Save ();
+                RaiseSubmissionUpdate (nb_tracks_scrobbled);
+            } else {
+                // TODO: If error == StationError.InvalidSessionKey,
+                // suggest to the user to (re)do the Last.fm authentication.
+                hard_failures++;
 
-                state = State.Idle;
+                queue.RemoveInvalidTracks ();
             }
+
+            // if there are still valid tracks in the queue then retransmit on the next interval
+            state = queue.Any () ? State.NeedTransmit : State.Idle;
         }
 
         private void LogIfIgnored (JsonObject scrobbled_track)
@@ -390,5 +444,29 @@ namespace Lastfm
             }
             current_now_playing_request = null;
         }
+
+        private void RaiseSubmissionStart (int total_count)
+        {
+            var handler = SubmissionStart;
+            if (handler != null) {
+                handler (this, new SubmissionStartEventArgs (total_count));
+            }
+        }
+
+        private void RaiseSubmissionUpdate (int update_count)
+        {
+            var handler = SubmissionUpdate;
+            if (handler != null) {
+                handler (this, new SubmissionUpdateEventArgs (update_count));
+            }
+        }
+
+        private void RaiseSubmissionEnd ()
+        {
+            var handler = SubmissionEnd;
+            if (handler != null) {
+                handler (this, null);
+            }
+        }
     }
 }
